@@ -42,6 +42,8 @@ READERS = {
     "gbq": pd.read_gbq,
     "google": pd.read_gbq,
     "bigquery": pd.read_gbq,
+    ### extras
+    "tsv": lambda fname: pd.read_csv(fname, sep="\t"),
 }
 
 
@@ -71,13 +73,79 @@ def pipein():
         return pd.DataFrame()
 
 
+@register
+def query(expr):
+    """Using pandas queries.
+
+    Usage: cat a.csv | ph query "x > 5"
+
+    """
+    df = pipein()
+    new_df = df.query(expr)
+    pipeout(new_df)
+
+
+@register
+def appendstr(col, s, newcol=None):
+    """Special method to append a string to the end of a column.
+
+    Usage: cat e.csv | ph appendstr year -01-01 | ph date year
+    """
+    df = pipein()
+    if newcol is None:
+        newcol = col
+    df[newcol] = df[col].astype(str) + s
+    pipeout(df)
+
+
+@register
+def astype(type, column=None, newcolumn=None):
+    """Cast a column to a different type.
+
+    Usage:  cat a.csv | ph astype double x [new_x]
+
+    """
+    df = pipein()
+    if column is None:
+        df = df.astype(type)
+    elif newcolumn is not None:
+        df[newcolumn] = df[column].astype(type)
+    else:
+        df[column] = df[column].astype(type)
+    pipeout(df)
+
+
+@register
+def monotonic(column, direction="+"):
+    """Check if a certain column is monotonically increasing or decreasing.
+
+    Usage:  cat a.csv | ph monotonic x
+            cat a.csv | ph monotonic x +  # equivalent to above
+            cat a.csv | ph monotonic x -  # for decreasing
+
+    """
+    df = pipein()
+    if column not in df:
+        exit("Unknown column {}".format(column))
+    if direction not in "+-":
+        exit("direction must be either + or -")
+    print("{}_monotonic".format(column))
+    if direction == "+":
+        print(df[column].is_monotonic)
+    else:
+        print(df[column].is_monotonic_decreasing)
+
+
 @register
 def plot(index=None):
     """Plot the csv file.
 
     Usage:  ph plot [index]
     """
-    import matplotlib.pyplot as plt
+    try:
+        import matplotlib.pyplot as plt
+    except ImportError:
+        exit("plot depends on matplotlib, install ph[plot]")
 
     df = pipein()
     if index is not None:
@@ -135,7 +203,7 @@ def tab():
 
 
 @register
-def tabulate(*args, **kwargs):
+def tabulate(*args):
     """Tabulate the output, meaning that the `ph` pipeline necessarily ends.
 
     Takes arguments --headers and --format=[grid, ...].
@@ -145,13 +213,16 @@ def tabulate(*args, **kwargs):
     """
     headers = tuple()
     fmt = None
+    index = True
+    if "--noindex" in args:
+        index = False
     if "--headers" in args:
         headers = "keys"
     for opt in args:
         if opt.startswith("--format="):
             fmt = opt.split("--format=")[1]
             break
-    print(tabulate_(pipein(), tablefmt=fmt, headers=headers))
+    print(tabulate_(pipein(), tablefmt=fmt, headers=headers, showindex=index))
 
 
 @register
@@ -192,8 +263,6 @@ def open(ftype, fname):
     """
     if ftype not in READERS:
         exit("Unknown filetype {}".format(ftype))
-    if not os.path.exists(fname):
-        exit("No such file {}".format(fname))
     reader = READERS[ftype]
     pipeout(reader(fname))
 
